/** @file screenmode.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2019.10.01

	@brief [ screen mode ]
*/


#include "screenmode.h"


ScreenModeBase::ScreenModeBase()
{
	disp_device_count = 0;
	screen_mode_count = 0;
	major_bits_per_pixel = _RGB888;
}

ScreenModeBase::~ScreenModeBase()
{
}

int ScreenModeBase::CountMode() const
{
	return screen_mode_count;
}

const screen_mode_t *ScreenModeBase::GetMode(int num) const
{
	if (0 <= num && num < screen_mode_count) {
		return &screen_mode[num];
	}
	return NULL;
}

int ScreenModeBase::FindMode(int disp_no, int width, int height) const
{
	int match = -1;
	for(int i = 0; i < screen_mode_count; i++) {
		if (screen_mode[i].disp_no == disp_no
		&&  screen_mode[i].width == width
		&&  screen_mode[i].height == height) {
			match = i;
			break;
		}
	}
	return match;
}

int ScreenModeBase::CountDisp() const
{
	return disp_device_count;
}

const disp_device_t *ScreenModeBase::GetDisp(int num) const
{
	if (0 <= num && num < disp_device_count) {
		return &disp_device[num];
	}
	return NULL;
}

void ScreenModeBase::GetDispSize(int num, int *width, int *height, int *bits_per_pixel) const
{
	if (width) *width = disp_device[num].re.w;
	if (height) *height = disp_device[num].re.h;
	if (bits_per_pixel) *bits_per_pixel = major_bits_per_pixel;
}

int ScreenModeBase::WithinDisp(int x, int y) const
{
	int match = -1;
	for (int disp_no = 0; disp_no < disp_device_count; disp_no++) {
		const VmRectWH *re = &disp_device[disp_no].re;

		if ((re->x <= x && x < (re->x + re->w - 96))
		 && (re->y <= y && y < (re->y + re->h - 96))) {
			match = disp_no;
			break;
		}
	}
	return match;
}

void ScreenModeBase::GetVirtualDispSize(int *width, int *height, int *bits_per_pixel) const
{
	int left = 0x7fffffff;
	int top = 0x7fffffff;
	int right = 0x80000000;
	int bottom = 0x80000000;
	for (int disp_no = 0; disp_no < disp_device_count; disp_no++) {
		const VmRectWH *re = &disp_device[disp_no].re;
		if (re->x < left) left = re->x;
		if (re->y < top) top = re->y;
		if ((re->x + re->w - 1) > right) right = (re->x + re->w - 1);
		if ((re->y + re->h - 1) > bottom) bottom = (re->y + re->h - 1);
	}
	if (width) {
		*width = right - left + 1;
		if (*width < 0) *width = 0;
	}
	if (height) {
		*height = bottom - top + 1;
		if (*height < 0) *height = 0;
	}
	if (bits_per_pixel) *bits_per_pixel = major_bits_per_pixel;
}
