/** @file screenmode.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2019.10.01

	@brief [ screen mode ]
*/

#ifndef SCREEN_MODE_H
#define SCREEN_MODE_H

#include "../common.h"
#include "../cchar.h"
#include "../res/resource.h"


#define DISP_DEVICE_MAX 3
typedef struct disp_device_st {
	CTchar   name;
	VmRectWH re;
} disp_device_t;

#define SCREEN_MODE_MAX (ID_SCREEN_FULLSCREEN24 - ID_SCREEN_FULLSCREEN1 + 1)
typedef struct screen_mode_st {
	int disp_no;
	int width;
	int height;
} screen_mode_t;


class ScreenModeBase
{
protected:
	int disp_device_count;
	disp_device_t disp_device[DISP_DEVICE_MAX];

	int screen_mode_count;
	screen_mode_t screen_mode[SCREEN_MODE_MAX];

	int major_bits_per_pixel;

public:
	ScreenModeBase();
	virtual ~ScreenModeBase();

	virtual void Enum() = 0;
	virtual void Enum(int desktop_width, int desktop_height, int bits_per_pixel) = 0;

	virtual int CountMode() const;
	virtual const screen_mode_t *GetMode(int num) const;

	virtual int FindMode(int disp_no, int width, int height) const;

	virtual int CountDisp() const;
	virtual const disp_device_t *GetDisp(int num) const;
	virtual void GetDispSize(int num, int *width, int *height, int *bits_per_pixel) const;

	virtual int WithinDisp(int x, int y) const;
	virtual void GetVirtualDispSize(int *width, int *height, int *bits_per_pixel) const;
};


#if defined(USE_WIN)
#include "windows/win_screenmode.h"
#elif defined(USE_SDL) || defined(USE_SDL2)
#include "SDL/sdl_screenmode.h"
#elif defined(USE_WX) || defined(USE_WX2)
#include "wxwidgets/wxw_screenmode.h"
#elif defined(USE_QT)
#include "qt/qt_screenmode.h"
#endif

#endif /* SCREEN_MODE_H */
