/** @file win_screenmode.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2019.10.01

	@brief [ screen mode ]
*/

#include "../screenmode.h"
#include "../../vm/vm_defs.h"
#include "../../logging.h"
//#include "win_main.h"


ScreenMode::ScreenMode()
	: ScreenModeBase()
{
	monitor_count = 0;
}

ScreenMode::~ScreenMode()
{
}

BOOL CALLBACK ScreenMode::MonitorProc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
	ScreenMode *obj = (ScreenMode *)dwData;
	return obj->get_monitor_device(hMonitor, hdcMonitor, lprcMonitor);
}

BOOL ScreenMode::get_monitor_device(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor)
{
	if (monitor_count < disp_device_count) {
			RECT re;
//			disp_device[monitor_count].hMoni = hMonitor;
			CopyRect(&re, lprcMonitor);
			RECT_IN(disp_device[monitor_count].re, re.left, re.top, re.right - re.left, re.bottom - re.top);
	}
	monitor_count++;

	return TRUE;
}

void ScreenMode::Enum()
{
	int w, h, bpp;
	GetDesktopSize(&w, &h, &bpp);
	Enum(w, h, bpp);
}

void ScreenMode::Enum(int desktop_width, int desktop_height, int bits_per_pixel)
{
	// enumerate display device
	int max_desktop_width = 0;
	int max_desktop_height = 0;
	disp_device_count = 0;
	for(int i = 0; disp_device_count < DISP_DEVICE_MAX; i++) {
		DISPLAY_DEVICE disp;
		ZeroMemory(&disp, sizeof(disp));
		disp.cb = sizeof(disp);
		if (!EnumDisplayDevices(NULL, i, &disp, 0)) {
			break;
		}
		if (disp.StateFlags & (DISPLAY_DEVICE_ACTIVE | DISPLAY_DEVICE_PRIMARY_DEVICE)) {
			disp_device[disp_device_count].name.Set(disp.DeviceName);
//			disp_device[disp_device_count].hMoni = NULL;
			disp_device_count++;
		}
	}
	major_bits_per_pixel = bits_per_pixel;

	// enumerate monitor size
	EnumDisplayMonitors(NULL, NULL, MonitorProc, (LPARAM)this);

	// enumerate screen mode for fullscreen
	int w,h,bpp;
	screen_mode_count = 0;
	for(int disp_no = 0; disp_no < disp_device_count && screen_mode_count < SCREEN_MODE_MAX; disp_no++) {
		for(int i = 0; screen_mode_count < SCREEN_MODE_MAX; i++) {
			DEVMODE dev;
			ZeroMemory(&dev, sizeof(dev));
			dev.dmSize = sizeof(dev);
			if(EnumDisplaySettings(disp_device[disp_no].name, i, &dev) == 0) {
				break;
			}
			w = dev.dmPelsWidth;
			h = dev.dmPelsHeight;
			bpp = dev.dmBitsPerPel;
			if(w < MIN_WINDOW_WIDTH || h < MIN_WINDOW_HEIGHT || w < 640 || h < 480) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			if(((w * 30 / h) != 40)
			&& ((w * 40 / h) != 50)
			&& ((w * 90 / h) != 160)
			&& ((w * 50 / h) != 80)) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			if (w <= 1280 && (w != 640 && w != 800 && w != 1024 && w != 1280)) {
				logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp ignored"), disp_no, w, h, bpp);
				continue;
			}
			bool found = false;
			for(int j = 0; j < screen_mode_count; j++) {
				if(screen_mode[j].disp_no == disp_no && screen_mode[j].width == w && screen_mode[j].height == h) {
					found = true;
					logging->out_debugf(_T("screen_mode:-- [%d] %dx%d %dbpp already exist"), disp_no, w, h, bpp);
					break;
				}
			}
			if(!found) {
				screen_mode[screen_mode_count].disp_no = disp_no;
				screen_mode[screen_mode_count].width = w;
				screen_mode[screen_mode_count].height = h;
				logging->out_debugf(_T("screen_mode:%2d [%d] %dx%d %dbpp"), screen_mode_count, disp_no, w, h, bpp);
				screen_mode_count++;

				if(max_desktop_width <= w) {
					max_desktop_width = w;
				}
				if(max_desktop_height <= h) {
					max_desktop_height = h;
				}
			}
		}
	}
	qsort(&screen_mode, screen_mode_count, sizeof(struct screen_mode_st), qsort_by_n_w_h);

	// if cannot get modes, add default screen size mode.
	if (screen_mode_count == 0) {
		screen_mode[screen_mode_count].disp_no = 0;
		screen_mode[screen_mode_count].width = desktop_width;
		screen_mode[screen_mode_count].height = desktop_height;
		screen_mode_count = 1;

		max_desktop_width = desktop_width;
		max_desktop_height = desktop_height;
	}
}

int ScreenMode::qsort_by_n_w_h(const void *a, const void *b)
{
	struct screen_mode_st *a1 = (struct screen_mode_st *)a;
	struct screen_mode_st *b1 = (struct screen_mode_st *)b;

	int rc = 0;
	int aa = a1->width * a1->height;
	int bb = b1->width * b1->height;
	if (!rc) rc = (a1->disp_no - b1->disp_no);
	if (!rc) rc = (bb - aa);
	return rc;
}

void ScreenMode::GetDesktopSize(int *width, int *height, int *bpp)
{
	// desktop size on the primary monitor
	HDC hdcScr = GetDC(NULL);
	if (width) *width = GetDeviceCaps(hdcScr, HORZRES);
	if (height) *height = GetDeviceCaps(hdcScr, VERTRES);
	if (bpp) *bpp = GetDeviceCaps(hdcScr, BITSPIXEL);
	ReleaseDC(NULL, hdcScr);
}
